@@ -77,27 +77,27 @@ static int nd_curl_debug(CURL *ch __attribute__((unused)),
     switch (type) {
     case CURLINFO_TEXT:
         buffer.assign(data, size);
-        nd_debug_printf("%s: %s", thread->GetTag().c_str(), buffer.c_str());
+        nd_dprintf("%s: %s", thread->GetTag().c_str(), buffer.c_str());
         break;
     case CURLINFO_HEADER_IN:
         buffer.assign(data, size);
-        nd_debug_printf("%s: <-- %s", thread->GetTag().c_str(), buffer.c_str());
+        nd_dprintf("%s: <-- %s", thread->GetTag().c_str(), buffer.c_str());
         break;
     case CURLINFO_HEADER_OUT:
         buffer.assign(data, size);
-        nd_debug_printf("%s: --> %s", thread->GetTag().c_str(), buffer.c_str());
+        nd_dprintf("%s: --> %s", thread->GetTag().c_str(), buffer.c_str());
         break;
     case CURLINFO_DATA_IN:
-        nd_debug_printf("%s: <-- %lu data bytes\n", thread->GetTag().c_str(), size);
+        nd_dprintf("%s: <-- %lu data bytes\n", thread->GetTag().c_str(), size);
         break;
     case CURLINFO_DATA_OUT:
-        nd_debug_printf("%s: --> %lu data bytes\n", thread->GetTag().c_str(), size);
+        nd_dprintf("%s: --> %lu data bytes\n", thread->GetTag().c_str(), size);
         break;
     case CURLINFO_SSL_DATA_IN:
-        nd_debug_printf("%s: <-- %lu SSL bytes\n", thread->GetTag().c_str(), size);
+        nd_dprintf("%s: <-- %lu SSL bytes\n", thread->GetTag().c_str(), size);
         break;
     case CURLINFO_SSL_DATA_OUT:
-        nd_debug_printf("%s: --> %lu SSL bytes\n", thread->GetTag().c_str(), size);
+        nd_dprintf("%s: --> %lu SSL bytes\n", thread->GetTag().c_str(), size);
         break;
     default:
         break;
@@ -134,13 +134,54 @@ static int ndSinkThread_progress(void *user,
     return 0;
 }
 
-ndSinkThread::ndSinkThread()
-    : ndThread("nd-sink", -1),
+ndSinkThread::ndSinkThread(int16_t cpu)
+    : ndThread("nd-sink", (long)cpu),
     headers(NULL), headers_gz(NULL), pending_size(0), post_errors(0),
     update_imf(1), update_count(0)
 {
+    CreateHandle();
+
     int rc;
 
+    pthread_condattr_t cond_attr;
+
+    pthread_condattr_init(&cond_attr);
+    pthread_condattr_setclock(&cond_attr, CLOCK_MONOTONIC);
+    if ((rc = pthread_cond_init(&uploads_cond, &cond_attr)) != 0)
+        throw ndSinkThreadException(strerror(rc));
+    pthread_condattr_destroy(&cond_attr);
+
+    if ((rc = pthread_mutex_init(&uploads_cond_mutex, NULL)) != 0)
+        throw ndSinkThreadException(strerror(rc));
+
+    if ((rc = pthread_mutex_init(&response_mutex, NULL)) != 0)
+        throw ndSinkThreadException(strerror(rc));
+}
+
+ndSinkThread::~ndSinkThread()
+{
+    pthread_cond_broadcast(&uploads_cond);
+
+    Join();
+
+    DestroyHandle();
+
+    pthread_cond_destroy(&uploads_cond);
+    pthread_mutex_destroy(&uploads_cond_mutex);
+
+    pthread_mutex_lock(&response_mutex);
+
+    for (ndResponseQueue::const_iterator i = responses.begin();
+        i != responses.end(); i++) delete (*i);
+
+    responses.clear();
+
+    pthread_mutex_unlock(&response_mutex);
+    pthread_mutex_destroy(&response_mutex);
+}
+
+void ndSinkThread::CreateHandle(void)
+{
     if ((ch = curl_easy_init()) == NULL)
         throw ndSinkThreadException("curl_easy_init");
 
@@ -187,50 +228,23 @@ ndSinkThread::ndSinkThread()
         curl_easy_setopt(ch, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1);
 
     CreateHeaders();
-
-    pthread_condattr_t cond_attr;
-
-    pthread_condattr_init(&cond_attr);
-    pthread_condattr_setclock(&cond_attr, CLOCK_MONOTONIC);
-    if ((rc = pthread_cond_init(&uploads_cond, &cond_attr)) != 0)
-        throw ndSinkThreadException(strerror(rc));
-    pthread_condattr_destroy(&cond_attr);
-
-    if ((rc = pthread_mutex_init(&uploads_cond_mutex, NULL)) != 0)
-        throw ndSinkThreadException(strerror(rc));
-
-    if ((rc = pthread_mutex_init(&response_mutex, NULL)) != 0)
-        throw ndSinkThreadException(strerror(rc));
 }
 
-ndSinkThread::~ndSinkThread()
+void ndSinkThread::DestroyHandle(void)
 {
-    pthread_cond_broadcast(&uploads_cond);
+    if (ch != NULL) {
+        curl_easy_cleanup(ch);
+        ch = NULL;
+    }
 
-    Join();
-
-    if (ch != NULL) curl_easy_cleanup(ch);
     FreeHeaders();
-
-    pthread_cond_destroy(&uploads_cond);
-    pthread_mutex_destroy(&uploads_cond_mutex);
-
-    pthread_mutex_lock(&response_mutex);
-
-    for (ndResponseQueue::const_iterator i = responses.begin();
-        i != responses.end(); i++) delete (*i);
-
-    responses.clear();
-
-    pthread_mutex_unlock(&response_mutex);
-    pthread_mutex_destroy(&response_mutex);
 }
 
 void *ndSinkThread::Entry(void)
 {
     int rc;
 
-    nd_debug_printf("%s: thread started.\n", tag.c_str());
+    nd_dprintf("%s: thread started.\n", tag.c_str());
 
     while (terminate == false) {
         Lock();
@@ -283,8 +297,10 @@ void ndSinkThread::Terminate(void)
 
     Lock();
 
-    if ((rc = pthread_cond_broadcast(&uploads_cond)) != 0)
+    if ((rc = pthread_cond_broadcast(&uploads_cond)) != 0) {
+        Unlock();
         throw ndSinkThreadException(strerror(rc));
+    }
 
     terminate = true;
 
@@ -303,8 +319,10 @@ void ndSinkThread::QueuePush(const string &json)
 
     uploads.push(json);
 
-    if ((rc = pthread_cond_broadcast(&uploads_cond)) != 0)
+    if ((rc = pthread_cond_broadcast(&uploads_cond)) != 0) {
+        Unlock();
         throw ndSinkThreadException(strerror(rc));
+    }
 
     Unlock();
 }
@@ -444,6 +462,9 @@ void ndSinkThread::Upload(void)
         nd_printf("%s: reverted to default sink URL: %s\n", tag.c_str(),
             nd_config.url_sink);
 
+        DestroyHandle();
+        CreateHandle();
+
         curl_easy_setopt(ch, CURLOPT_URL, nd_config.url_sink);
 
         post_errors = 0;
@@ -452,13 +473,13 @@ void ndSinkThread::Upload(void)
     if (++update_count == update_imf || post_errors > 0)
         update_count = 0;
     else {
-        nd_debug_printf("%s: payload upload delay: %u of %u\n",
+        nd_dprintf("%s: payload upload delay: %u of %u\n",
             tag.c_str(), update_count, update_imf);
         return;
     }
 
     do {
-        nd_debug_printf("%s: payload %lu/%lu (%d of %d bytes)...\n",
+        nd_dprintf("%s: payload %lu/%lu (%d of %d bytes)...\n",
             tag.c_str(), ++xfer, total, pending.front().second.size(), pending_size);
 
         if (! pending.front().first)
@@ -508,28 +529,15 @@ void ndSinkThread::Upload(void)
         char *content_type = NULL;
         curl_easy_getinfo(ch, CURLINFO_CONTENT_TYPE, &content_type);
 
-        double content_length = 0;
+        double content_length = 0.0f;
         curl_easy_getinfo(ch, CURLINFO_CONTENT_LENGTH_DOWNLOAD, &content_length);
 
-        if (content_type != NULL && content_length != 0.0f) {
-            if (strcasecmp("application/json", content_type) == 0)
-                ProcessResponse();
-            else {
-                ndJsonResponse *response = new ndJsonResponse(
-                    ndJSON_RESP_INVALID_CONTENT_TYPE,
-                    "Invalid content type, expected: application/json"
-                );
+        if (content_type == NULL) {
+            nd_dprintf("%s: Missing content type.\n", tag.c_str());
 
-                if (response == NULL)
-                    throw runtime_error(strerror(ENOMEM));
-                else
-                    PushResponse(response);
-            }
-        }
-        else {
             ndJsonResponse *response = new ndJsonResponse(
-                ndJSON_RESP_INVALID_RESPONSE,
-                "Unknown content type or invalid content length"
+                ndJSON_RESP_INVALID_CONTENT_TYPE,
+                "Missing content type, expected: application/json"
             );
 
             if (response == NULL)
@@ -537,6 +545,35 @@ void ndSinkThread::Upload(void)
             else
                 PushResponse(response);
         }
+        else if (content_length == 0.0f) {
+            nd_dprintf("%s: Zero-length content length.\n", tag.c_str());
+
+            ndJsonResponse *response = new ndJsonResponse(
+                ndJSON_RESP_INVALID_RESPONSE,
+                "Invalid content length (zero-bytes)"
+            );
+
+            if (response == NULL)
+                throw runtime_error(strerror(ENOMEM));
+            else
+                PushResponse(response);
+        }
+        else if (strcasecmp("application/json", content_type) != 0) {
+
+            nd_dprintf("%s: Unexpected content type.\n", tag.c_str());
+
+            ndJsonResponse *response = new ndJsonResponse(
+                ndJSON_RESP_INVALID_CONTENT_TYPE,
+                "Invalid content type, expected: application/json"
+            );
+
+            if (response == NULL)
+                throw runtime_error(strerror(ENOMEM));
+            else
+                PushResponse(response);
+        }
+        else
+            ProcessResponse();
 
         switch (http_rc) {
         case 200:
@@ -552,7 +589,7 @@ void ndSinkThread::Upload(void)
                     fwrite(pending.front().second.data(),
                         1, pending.front().second.size(), hf);
                     fclose(hf);
-                    nd_debug_printf(
+                    nd_dprintf(
                         "%s: wrote rejected payload to: %s\n",
                         tag.c_str(), ND_JSON_FILE_BAD_SEND);
                 }
@@ -622,7 +659,7 @@ string ndSinkThread::Deflate(const string &data)
 
     if (ND_DEBUG || ND_DEBUG_UPLOAD) {
 
-        nd_debug_printf(
+        nd_dprintf(
             "%s: payload compressed: %lu -> %lu: %.1f%%\n",
             tag.c_str(), data.size(), buffer.size(),
             100.0f - ((float)buffer.size() * 100.0f / (float)data.size())
@@ -688,7 +725,7 @@ void ndSinkThread::ProcessResponse(void)
         }
 
         if (response->update_imf > 0 && response->update_imf != update_imf) {
-            nd_debug_printf("%s: changing update multiplier from: %u"
+            nd_dprintf("%s: changing update multiplier from: %u"
                 " to: %u\n", tag.c_str(), update_imf, response->update_imf);
             update_imf = response->update_imf;
         }
@@ -712,7 +749,7 @@ void ndSinkThread::ProcessResponse(void)
     }
 
     if (response != NULL) {
-        nd_debug_printf("%s: [%d] %s\n", tag.c_str(),
+        nd_dprintf("%s: [%d] %s\n", tag.c_str(),
             response->resp_code,
             (response->resp_message.size() > 0) ?
                 response->resp_message.c_str() : "(no message)");
